#ifndef CGAL_DE_BOOR_FO_H
#define CGAL_DE_BOOR_FO_H

#include "Types.h"
#include "Curve_constants.h"
#include "Helpers.h"


// Template function object class which implements the de Boor algorithm
// for calculating a point on a b-spline curve at given parameter value t
// Parameters:
//    NumType_    the number type of the points coordinates 
template <class NumType_> 
class DeBoor_FO {
public:
    typedef NumType_ NumType;
    typedef vector<NumType> ParamVector;
    typedef CGAL::Cartesian<NumType> REP;
    typedef CGAL::Point_2<REP> Point;   
    typedef vector<Point> Points;
    typedef CGAL::Vector_2<REP> Vector;
    typedef vector<Vector> Vectors;

    DeBoor_FO() : contr_pts(NULL), knots(NULL), d(0) {
    }
    
    DeBoor_FO(Points * contr_points, ParamVector * param, 
	     unsigned degree) : 
	contr_pts(contr_points), knots(param), d(degree) {	
    }    

    // operator(): NumType -> Point
    // Parameters:
    //   t    the parameter value 
    // Pre: a<=t<=b
    //      there is at least one control point
    // Returns: a point on the B-Spline curve calculated with the DeBoor's
    //          insertion algorithm at t
    Point operator() (NumType t) const {
	assert(!(*contr_pts).empty());
	assert(!(*knots).empty() && (*knots).size()>1);	
	assert((*contr_pts).size()==(*knots).size());
	unsigned number = (*knots).size()-1;
	assert(d<=number);
	assert((*knots)[0]<=t && t<=(*knots)[(*knots).size()-1]);
	unsigned k = 0;
	//find k so that t_k > t in [t_k-1,t_k) 
	while((k<number) && (t>=(*knots)[k])) k++;
	if(k==0) return (* contr_pts)[0];
	if(k==number && t==(*knots)[number]) return (*contr_pts)[number];
	
	Vectors tmp_contr_pts;	
	ParamVector tmp_knots;      
	//index of the first neended control point
	int contr_pt_index;
	if(d%2==0) {
	    contr_pt_index= k-(d/2);
	}
	else contr_pt_index= k-(d/2)-1;
	//index of the first needed knot
	int knot_index = k-d;
	
	for(int j=0; j<=d; j++) {
	    if(contr_pt_index<0) 
		tmp_contr_pts.push_back((* contr_pts)[0] - CGAL::ORIGIN);
	    else {
		if(contr_pt_index>number) 
		    tmp_contr_pts.push_back((*contr_pts)[number] - 
					CGAL::ORIGIN);
		else tmp_contr_pts.push_back((* contr_pts)[contr_pt_index] - 
					     CGAL::ORIGIN);
	    }
	    contr_pt_index++;	    
	}
	//the number of needed knots is d+1+1+..+1(d times)=d+1*d=2*d
	for(int j=0; j<2*d; j++) {
	    if(knot_index<0) tmp_knots.push_back((* knots)[0]);
	    else if(knot_index>number) 
		tmp_knots.push_back((* knots)[number]);
	    else tmp_knots.push_back((* knots)[knot_index]);
	    knot_index++;
	}
       
	NumType denom;
	for(int j = d; j>0; j--) {	
	    for(int k = 0; k<j; k++) {
		denom= tmp_knots[k+d] - tmp_knots[d-j+k];		
		if(denom==0.0) {
		    tmp_contr_pts[k] = tmp_contr_pts[k];
		}
		//P[k]=((t[k+d]-t)*P[k]+(t-t[k])*P[k+1])/(t[k+d]-t[k]);
		else
		    tmp_contr_pts[k] =(tmp_contr_pts[k]*(tmp_knots[k+d]-t)
			 + tmp_contr_pts[k+1]*(t-tmp_knots[d-j+k]))/denom;
	    }
	}
	return (CGAL::ORIGIN + tmp_contr_pts[0]);		
    }

    // set_degree: unsigned -> void
    // Effect:     sets degree to d
    void set_degree(unsigned degree) {
	d = degree;
    }
private:
    Points * contr_pts;
    ParamVector * knots;
    unsigned d;

};
#endif //CGAL_DE_BOOR_FO_H
