#ifndef CGAL_DE_CASTELJAU_FO_H
#define CGAL_DE_CASTELJAU_FO_H

#include "Types.h"
#include "Curve_constants.h"
#include "Helpers.h"

// Template function object class which implements the de Casteljau algorithm
// for calculating a point on a Bezier curve at given parameter value t
// Parameters:
//    NumType_    the number type of the points coordinates 
template <class NumType_> 
class DeCasteljau_FO {
public:
    typedef NumType_ NumType;
    typedef CGAL::Cartesian<NumType> REP;
    typedef CGAL::Point_2<REP> Point;   
    typedef vector<Point> Points;
    typedef CGAL::Vector_2<REP> Vector;
    typedef vector<Vector> Vectors;

    DeCasteljau_FO() : contr_pts(NULL), a(LOW_BOUND), b(UP_BOUND) {
    }
    
    DeCasteljau_FO(Points * contr_points, NumType a_=LOW_BOUND, 
		   NumType b_=UP_BOUND) : 
      contr_pts(contr_points), a(a_), b(b_) {
    }    

    // operator(): NumType -> Point
    // Parameters:
    //   t    the parameter value    
    // Pre: a<=t<=b
    //      there is at least one control point
    // Returns: a point on the Bezier curve calculated with the de Casteljau 
    //          algorithm at t
    Point operator() (NumType t) const {
	assert(!(*contr_pts).empty());
	assert(a<=t && t<=b);
	int n = (*contr_pts).size()-1;	
	if(t==a) return (* contr_pts)[0];	
	if(t==b) return (* contr_pts)[n];
	Vectors tmp_v(n+1);
	for(int i=0; i<=n; i++) {
	    tmp_v[i] = (* contr_pts)[i] - CGAL::ORIGIN;
	}
	NumType b_t_coef = (b-t)/(b-a);
	NumType a_t_coef = (t-a)/(b-a);	
	for(int j = (n-1); j>=0; j--) {
	    for(int k = 0; k<=j; k++) {
		//b[k]=((b-t)*b[k] +(t-a)*b[k+1])/(b-a);
		tmp_v[k] = tmp_v[k]*b_t_coef + tmp_v[k+1]*a_t_coef;	
	    }
	}
	return (CGAL::ORIGIN + tmp_v[0]);		
    }
    
private:
    Points * contr_pts;
    NumType a, b;
};
#endif //CGAL_DE_CASTELJAU_FO_H
